System vaVersion = 16r0300	ifTrue: [ 

"If you have version 4.0 please import the application EnhancedDebuggerApp from the 
file ENHNDBBG.DAT"

Application create: #EnhancedDebuggerApp with:
    (#( EtTools)
        collect: [:each | Smalltalk at: each ifAbsent: [
            Application errorPrerequisite: #EnhancedDebuggerApp missing: each]])!

EnhancedDebuggerApp becomeDefault!
EtDebugger subclass: #EnhancedDebugger
    instanceVariableNames: 'throughButtonWidget '
    classVariableNames: ''
    poolDictionaries: ''!

EnhancedDebugger comment: 'This code was written by Joe Winchester and Mark Jones.  
email 103276,233@compuserve.com 	or 	JoeW@concentric.net 	or		 MarkJ@ibm.net

This code is provided with no guarantee of its reliability and any defects or problems
arising from its use are not the responsibility of the authors.  Evaluating the following expression will
reset the image to use the IBM supplied debugger.

System startUpClass debuggerClass: EtDebugger.

To active the enhanced debugger evaluate.

System startUpClass debuggerClass: EnhancedDebugger.
'!

EnhancedDebuggerApp becomeDefault!
Application subclass: #EnhancedDebuggerApp
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!


EnhancedDebuggerApp becomeDefault!

!EnhancedDebugger publicMethods !

createWorkRegion

		super createWorkRegion.

		throughButtonWidget := self
			newButtonWidget: 'Through'
			selected: #processThrough.
				
		self stepIntoButtonWidget setValuesBlock: [ :w |
			w rightWidget: throughButtonWidget ].

		throughButtonWidget setValuesBlock: [ :w |
			w
				topAttachment: XmATTACHOPPOSITEWIDGET;
				topWidget: self stepToReturnButtonWidget;
				leftAttachment: XmATTACHPOSITION;
				leftPosition: 11;
				rightAttachment: XmATTACHWIDGET;
				rightWidget: self stepOverButtonWidget;
				bottomAttachment: XmATTACHWIDGET;
				bottomWidget: self textWidget parent].

		self stepOverButtonWidget setValuesBlock: [ :w |
			w leftPosition: 22 ].

		self stepToReturnButtonWidget setValuesBlock: [ :w |
			w leftPosition: 33 ].

		self resumeButtonWidget setValuesBlock: [ :w |
			w leftPosition: 44 ].

!

currentParseNode

		^self parseTree == nil ifFalse: [
			self parseTree 
			nodeWhichContainsPC: (self currentPC: self selectedFrameIndex )
			hasDropped: self selectedProcess hasDropped ]
		
!

label

	"Answer the label for the debugger."

	^'Enhanced Debugger.  Joe Winchester and Mark Jones'!

processStepThrough

		| sem currentProcess currentMethod |

		"don't continue if selected process is not resumable"
		self isSelectedProcessResumable ifFalse: [ ^self ].

		"don't continue if a source change has been specified"
		self changeRequest ifFalse: [ ^self ].

		"ensure there is a selected method"
		self selectTopFrameIfNone.
		currentProcess := self selectedProcess.
		currentMethod := self selectedMethod.

		"Break every byte code of the process and resume execution until the
		method is active once more or is not in the method stack"
		currentProcess
			breakEveryBytecode: true ;
			useBytecodeMask: true.
		sem := Semaphore new.
		Processor
			debugResume: currentProcess
			when: [ 
				currentProcess activeMethod == currentMethod or: [ 
				currentProcess cannotReturnTo: currentMethod ] ]
			do: [ :hasStackOverflowOccured |
				hasStackOverflowOccured
					ifTrue: [ self removeProcess: currentProcess ].
				sem signal ].
		sem wait.

		"refresh debugger"
		self isOneProcessSelected ifTrue: [ self refreshAfterStep ].




!

processThrough

		| aNode |

		aNode := self currentParseNode.

		"process as through if selected method is the active method"
		(self selectedFrameIndex == self selectedProcess startFrame

		"and, if node requires through"
		and: [ aNode notNil and: [ aNode isThroughRequired ]])
			ifTrue: [ self processStepThrough ]
			ifFalse: [ self processStepOver ]

! !


!EnhancedDebuggerApp class publicMethods !

addToCopyrightText

	"Get the copyright text and add to it"

	self
		addToPrimitiveBaseApp ;
		addToNlsCatKRN!

addToNlsCatKRN

	"Get the copyright text and add to it"

	| copyrightAssociation |	
	
	copyrightAssociation := NlsCatKRN associationAt: 'MxKRN9' ifAbsent: [].
	copyrightAssociation == nil ifFalse: [ 
		| copyrightText |
		copyrightText := copyrightAssociation value.
		( copyrightText indexOfSubCollection: self debuggerCopyrightText startingAt: 1 ) == 0 ifTrue: [ 
		copyrightText := self debuggerCopyrightText , copyrightText.
		copyrightAssociation value: copyrightText ] ].
	
!

addToPrimitiveBaseApp

	"Add the copyright string in the primitive base app"

	| copyrightText |	
	
	copyrightText := AbtPrimitiveBaseApp applicationCopyright.
	( copyrightText indexOfSubCollection: self debuggerCopyrightText startingAt: 1 ) == 0 ifTrue: [ 
	copyrightText := self debuggerCopyrightText , copyrightText.
	AbtPrimitiveBaseApp applicationCopyright: copyrightText ].
!

debuggerCopyrightText 

	"Return a string of copyright text"

	^'Enhanced Debugger V 1.0 1996\By Joe Winchester and Mark Jones\'!

loaded

	System startUpClass debuggerClass: EnhancedDebugger.
	self 
		addToCopyrightText ;
		writeInstructionsToTranscript ;
		writeExampleToTranscript.!

removeFromCopyrightText

	"Get the copyright text and remove from it"

	self
		removeFromPrimitiveBaseApp ;
		removeFromNlsCatKRN!

removeFromNlsCatKRN

	"Get the copyright text and remove from it"

	| copyrightAssociation |	
	
	copyrightAssociation := NlsCatKRN associationAt: 'MxKRN9' ifAbsent: [].
	copyrightAssociation == nil ifFalse: [ 
		| copyrightText preceedingText followingText indexOfDebuggerCopyright |
		copyrightText := copyrightAssociation value.
		indexOfDebuggerCopyright := copyrightText indexOfSubCollection: self debuggerCopyrightText startingAt: 1.
		indexOfDebuggerCopyright == 0 ifFalse: [ 
			preceedingText := indexOfDebuggerCopyright = 1
					ifTrue: [ '' ]
					ifFalse: 	[ copyrightText copyFrom: 1 to: indexOfDebuggerCopyright - 1 ].
			followingText := copyrightText 
				copyFrom: indexOfDebuggerCopyright + self debuggerCopyrightText size 
				to: copyrightText size.
			copyrightAssociation value: preceedingText , followingText ] ].!

removeFromPrimitiveBaseApp 

	"Remove the copyright text from the class variable on the primitive base app"

	| copyrightText preceedingText followingText indexOfDebuggerCopyright |

	copyrightText := 	AbtPrimitiveBaseApp applicationCopyright.
	indexOfDebuggerCopyright := copyrightText indexOfSubCollection: self debuggerCopyrightText startingAt: 1.
	indexOfDebuggerCopyright == 0 ifFalse: [ 
		preceedingText := indexOfDebuggerCopyright = 1
				ifTrue: [ '' ]
				ifFalse: 	[ copyrightText copyFrom: 1 to: indexOfDebuggerCopyright - 1 ].
		followingText := copyrightText 
			copyFrom: indexOfDebuggerCopyright + self debuggerCopyrightText size 
			to: copyrightText size.
		AbtPrimitiveBaseApp applicationCopyright: preceedingText , followingText ].!

removing

	System startUpClass debuggerClass: EtDebugger.
	self removeFromCopyrightText.
!

writeExampleToTranscript

	Transcript cr ; cr ;
		show: 'Try debugging the following method to see its effectiveness' ; cr ; cr ;
		show: 
'	| a b |
	self halt.
	[ System showBusyCursorWhile: [ 
		a := Dictionary new.
		b := OrderedCollection with: 1 with: 2.
		a at: 1 ifAbsentPut: [ Date today ].
		a keysAndValuesDo: [ :key :value |
		( key = 1 and: [ value = Date today ] ) ifTrue: [
			( b inject: 0 into: [ :total :elem | total + elem ] ) printString ] ] ]
	] when: ExError do: [ :aSignal | aSignal exitWith: nil ] '  ;
		
		cr ; cr ;
		show: 'Use just the through button to see every message send within ' ; cr ;
		show: 'the source visited, each block returned from straight to the original ' ; cr ;
		show: 'method and the loops iterated around directly' ; cr.!

writeInstructionsToTranscript

	Transcript cr ;
		show: 'To switch to the enhanced debugger evaluate ' ; cr ;
		tab ; show: 'System startUpClass debuggerClass: EnhancedDebugger' ; cr ;
		show: 'To switch back to the supplied debugger evaluate ' ; cr ;
		tab ; show: 'System startUpClass debuggerClass: EtDebugger' ; cr ;
		show: 'This code was written by Joe Winchester and Mark Jones.  It introduces a new button ' ; cr ; 
		show: 'labelled through on the debugger.  Through can be used entirely instead of over and ' ; cr ;
		show: 'will visit every message send within the source being debugged, even when it is in ' ; cr ;
		show: 'a block.  It is at its best with collection iteration methods or any method where source ' ;cr ; 
		show: 'is inside a block.  With non block source it behaves exactly as over used to'  ; cr.
		! !


!EsBlock publicMethods !

isThroughRequired

	^true! !


!EsKeywordPattern publicMethods !

isThroughRequired

		self arguments
			detect: [ :anArgument |
				anArgument isThroughRequired ]
			ifNone: [ ^false ].
		^true
! !


!EsMessageExpression publicMethods !

isThroughRequired

		^self receiver isThroughRequired
			or: [ self messagePattern isThroughRequired ]
! !


!EsParseNode publicMethods !

isThroughRequired
	
		^false
! !


!Process publicMethods !

activeMethod

	^self methodAtFrame: self startFrame!

cannotReturnTo: aMethod

	"Loop around the frames and get the method at each.  Return false
	if any of these are the method argument"

		self startFrame to: self numberOfFrames - 1 do: [ :index |
			( self methodAtFrame: index ) == aMethod
				ifTrue: [ ^false ] ].
		^true
! !


EnhancedDebugger initializeAfterLoad!
EnhancedDebuggerApp initializeAfterLoad!

EnhancedDebuggerApp loaded!

]
